# Copyright 2018 miruka
# This file is part of harmonyqt, licensed under GPLv3.

"""Test Matrix client"""

__pkg_name__ = "harmonyqt"
__version__  = "0.5.8"
__status__   = "Development"
# __status__ = "Production"

__author__   = "miruka"
__email__    = "miruka@disroot.org"

__license__  = "GPLv3"
